@@ -1,8 +1,8 @@
 package HTML::Template::Compiled;
-# $Id: Compiled.pm,v 1.301 2006/12/12 21:52:18 tinita Exp $
+# $Id: Compiled.pm 1018 2008-03-01 01:34:49Z tinita $
 # doesn't work with make tardist
 #our $VERSION = ($version_pod =~ m/^\$VERSION = "(\d+(?:\.\d+)+)"/m) ? $1 : "0.01";
-our $VERSION = "0.83_01";
+our $VERSION = "0.91_001";
 use Data::Dumper;
 BEGIN {
 use constant D => $ENV{HTC_DEBUG} || 0;
@@ -10,10 +10,11 @@ use constant D => $ENV{HTC_DEBUG} || 0;
 use strict;
 use warnings;
 
+our $Storable = 1;
 use Carp;
 use Fcntl qw(:seek :flock);
 use File::Spec;
-use File::Basename qw(dirname);
+use File::Basename qw(dirname basename);
 use HTML::Template::Compiled::Utils qw(:walkpath :log :escape &md5);
 use HTML::Template::Compiled::Expression qw(:expressions);
 use HTML::Template::Compiled::Compiler;
@@ -22,13 +23,18 @@ eval {
     require HTML::Entities;
     require URI::Escape;
 };
+eval {
+    require Encode;
+};
+my $Encode = $@ ? 0 : 1;
+
 use base 'Exporter';
 our @EXPORT_OK = qw(&HTC);
 use HTML::Template::Compiled::Parser qw(
     $CASE_SENSITIVE_DEFAULT $NEW_CHECK
     $DEBUG_DEFAULT $SEARCHPATH
-    %FILESTACK $DEFAULT_ESCAPE $DEFAULT_QUERY
-    $UNTAINT $DEFAULT_TAGSTYLE
+    %FILESTACK %COMPILE_STACK %PATHS $DEFAULT_ESCAPE $DEFAULT_QUERY
+    $UNTAINT $DEFAULT_TAGSTYLE $MAX_RECURSE
 );
 use vars qw($__ix__);
 
@@ -44,12 +50,13 @@ BEGIN {
     my @map = (
         undef, qw(
           path filename file scalar filehandle
-          cache_dir cache search_path
+          file_cache cache_dir cache search_path
           loop_context case_sensitive global_vars
           default_path
-          debug perl out_fh default_escape
+          debug debug_file objects perl out_fh default_escape
           filter formatter
-          globalstack use_query parser compiler includes
+          globalstack use_query parse_tree parser compiler includes
+          plugins open_mode
         )
     );
 
@@ -143,6 +150,7 @@ sub new_from_perl {
     my $self = bless [], $class;
     D && $self->log("new(perl) filename: $args{filename}");
 
+    $self->init_cache(\%args);
     $self->init(%args);
     $self->set_perl( $args{perl} );
     $self->set_cache( exists $args{cache} ? $args{cache} : 1 );
@@ -173,7 +181,8 @@ sub new_file {
     }
     $self->set_filename( $filename );
     $self->set_cache( exists $args{cache} ? $args{cache} : 1 );
-    $self->set_cache_dir( $args{cache_dir} );
+    $self->init_cache(\%args);
+    #$self->set_cache_dir( $args{cache_dir} );
     $self->set_path( $args{path} );
     if (my $t = $self->from_cache()) {
         $t->init_includes;
@@ -197,7 +206,8 @@ sub new_filehandle {
     $args{path} = $self->build_path($args{path});
     $self->set_filehandle( $args{filehandle} );
     $self->set_cache(0);
-    $self->set_cache_dir( $args{cache_dir} );
+    $self->init_cache(\%args);
+    #$self->set_cache_dir( $args{cache_dir} );
     $self->set_path( $args{path} );
     if (my $t = $self->from_cache()) {
         return $t;
@@ -230,7 +240,8 @@ sub new_scalar_ref {
     $args{scalarref} = $scalarref;
     $args{path} = $self->build_path($args{path});
     $self->set_cache( exists $args{cache} ? $args{cache} : 1 );
-    $self->set_cache_dir( $args{cache_dir} );
+    $self->init_cache(\%args);
+    #$self->set_cache_dir( $args{cache_dir} );
     $self->set_scalar( $args{scalarref} );
     my $text = $self->get_scalar;
     my $md5  = md5($$text);
@@ -406,7 +417,7 @@ sub from_cache {
         return unless -d $dir;
         ref $self and $self->lock;
         opendir my $dh, $dir or die "Could not open '$dir': $!";
-        my @files = grep { m/\.pl$/ } readdir $dh;
+        my @files = grep { m/(\.pl|\.storable)$/ } readdir $dh;
         for my $file (@files) {
             my $file = File::Spec->catfile( $dir, $file );
             unlink $file or die "Could not delete '$file': $!";
@@ -456,17 +467,11 @@ sub compile {
     my $compiler = $self->get_compiler;
     if ( my $file = $self->get_file and !$self->get_scalar ) {
 
-        # thanks to sam tregars testsuite
-        # don't recursively include
-        my $recursed = ++$FILESTACK{$file};
         D && $self->log( "compile from file " . $file );
         die "Could not open '$file': $!" unless -f $file;
         my @times = $self->_checktimes($file);
         my $text  = $self->_readfile($file);
-        die "HTML::Template: recursive include of " . $file . " $recursed times"
-          if $recursed > 10;
         my ( $source, $compiled ) = $compiler->compile( $self, $text, $file );
-        --$FILESTACK{$file} or delete $FILESTACK{$file};
         $self->set_perl($compiled);
         $self->get_cache and $self->add_mem_cache(
             checked => time,
@@ -514,14 +519,34 @@ sub add_file_cache {
     my $filename = $self->get_filename;
     my $lmtime   = localtime $times{mtime};
     my $lchecked = localtime $times{checked};
-    D && $self->log("add_file_cache() $cache/$plfile");
-    open my $fh, ">$cache/$plfile.pl" or die $!;    # TODO File::Spec
+    my $cachefile = "$cache/$plfile";
+    D && $self->log("add_file_cache() $cachefile");
+    if ($Storable and require_storable()) {
+        #require Storable;
+        local $Storable::Deparse = 1;
+        my $clone = $self->clone;
+        $clone->prepare_for_cache;
+        my $v = $self->VERSION || '0.01';
+        my $to_cache = {
+            htc => $clone,
+            version => $v,
+            times => {
+                mtime => $times{mtime},
+                checked => $times{checked},
+            },
+        };
+        Storable::store($to_cache, "$cachefile.storable");
+    }
+    else {
+    my $utf8 = $Encode && Encode::is_utf8($source);
+    my $use_utf8 = $utf8 ? 'use utf8;' : '';
+    open my $fh, ">" . ($utf8 ? ':utf8' : ''), "$cachefile.pl" or die $!;    # TODO File::Spec
     my $path     = $self->get_path;
     my $path_str = '['
       . ( join ', ', map { $self->quote_file($_) } @$path )
       . ']';
     my $isScalar = $self->get_scalar ? 1 : 0;
-    my $query_info = $self->get_use_query;
+    my $query_info = $self->get_parse_tree;
     $query_info = Data::Dumper->Dump([$query_info], ['query_info']);
     my $parser =$self->get_parser;
     $parser = Data::Dumper->Dump([\$parser], ['parser']);
@@ -536,6 +561,12 @@ sub add_file_cache {
     #$includes_to_string =~ s/\$includes = //;
     my $search_path = $self->get_search_path || 0;
     my $gl = $self->get_global_vars;
+    my $debug_file = $self->get_debug_file;
+    $debug_file =~ tr/a-z0,//cd;
+    my $use_objects = $self->get_objects;
+    $use_objects =~ tr/a-z0//cd;
+    my $plugins = $self->get_plugins;
+    my $plugin_dump = Data::Dumper->Dump([\@$plugins], ['plugin_dump']);
     my $file_args = $isScalar
       ? <<"EOM"
         scalarref => $isScalar,
@@ -545,11 +576,15 @@ EOM
         filename => '@{[$self->get_filename]}',
 EOM
     my $package = <<"EOM";
+    $use_utf8
     package HTML::Template::Compiled;
 # file date $lmtime
 # last checked date $lchecked
+# @$plugins
+my $plugin_dump;
 my $query_info;
-my $parser;
+my \$parser;
+$parser;
 my $includes_to_string;
 my \$args = {
     # HTC version
@@ -572,8 +607,11 @@ $file_args
     use_query => \$query_info,
     parser => \$parser,
     global_vars => $gl,
+    debug_file => '$debug_file',
+    objects => '$use_objects',
     includes => \$includes,
     search_path_on_include => $search_path,
+    plugin => \$plugin_dump,
     # TODO
     # dumper => ...
     # template subroutine
@@ -583,16 +621,25 @@ $file_args
 EOM
     print $fh $package;
     D && $self->log("$cache/$plfile.pl generated");
+    }
     $self->unlock;
 }
 
+sub get_plugin {
+    my ($self, $class) = @_;
+    for my $plug (@{ $self->get_plugins || [] }) {
+        return $plug if (ref $plug || $plug) eq $class;
+    }
+    return;
+}
+
 sub from_file_cache {
     my ($self, $dir, $file) = @_;
     D && $self->stack;
     D && $self->log("include file: $file");
 
     my $escaped = $self->escape_filename($file);
-    my $req     = File::Spec->catfile( $dir, "$escaped.pl" );
+    my $req     = File::Spec->catfile( $dir, "$escaped.".($Storable && require_storable()?"storable":"pl") );
     return unless -f $req;
     return $self->include_file($req);
 }
@@ -601,17 +648,49 @@ sub include_file {
     my ( $self, $req ) = @_;
     D && $self->log("do $req");
     my $r;
+    my $t;
+    if ($Storable && require_storable()) {
+        #require Storable;
+        local $Storable::Eval = 1;
+        my $cache;
+        eval {
+            $cache = Storable::retrieve($req);
+        };
+        return if $@;
+        my $cached_version = $cache->{version};
+        $t = $cache->{htc};
+        if (($t->VERSION || '0.01') ne $cached_version || !$t->uptodate( $cache->{times} )) {
+            # is not uptodate
+            return;
+        }
+        my $plug = $t->get_plugins || [];
+        $t->init_plugins(@$plug);
+		$t->get_cache and $t->add_mem_cache(
+			checked => $cache->{times}->{checked},
+			mtime   => $cache->{times}->{mtime},
+		);
+    }
+    else {
     if ($UNTAINT) {
         # you said explicitly that you can trust your compiled code
-        open my $fh, '<', $req or die "Could not open '$req': $!";
-        local $/;
+        open my $fh, '<:utf8', $req or die "Could not open '$req': $!";
         my $code = <$fh>;
+        if ($code =~ m/use utf8/) {
+            binmode $fh, ':utf8';
+            $Encode and Encode::_utf8_on($code);
+        }
+        local $/;
+        $code .= <$fh>;
+        my $utf8 = $Encode && Encode::is_utf8($code);
         if ( $code =~ m/(\A.*\z)/ms ) {
             $code = $1;
         }
         else {
             $code = "";
         }
+        if ($utf8) {
+            Encode::_utf8_on($code);
+        }
         $r = eval $code;
     }
     else {
@@ -625,7 +704,7 @@ sub include_file {
     my $class = $r->{class} || 'HTML::Template::Compiled';
     my $args = $r->{htc};
     # we first just create from cached perl-code
-    my $t = $class->new_from_perl(%$args);
+    $t = $class->new_from_perl(%$args);
     if ($VERSION ne $cached_version || !$t->uptodate( $r->{times} )) {
         # is not uptodate
         return;
@@ -636,6 +715,7 @@ sub include_file {
         checked => $r->{times}->{checked},
         mtime   => $r->{times}->{mtime},
     );
+    }
     return $t;
 }
 
@@ -643,22 +723,48 @@ sub createFilename {
     my ( $self, $path, $filename ) = @_;
     D && $self->log("createFilename($path,$filename)");
     D && $self->stack(1);
-    if ( !$path or !length $path or
+    if ($path) {
+        local $" = "\\";
+        my $cached = $PATHS{"@$path"}->{$filename};
+        return $cached if defined $cached;
+    }
+    if ( !$path or
         (File::Spec->file_name_is_absolute($filename) &&
         -f $filename) ) {
         return $filename;
     }
     else {
         D && $self->log( "file: " . File::Spec->catfile( $path, $filename ) );
-        my @paths = ref $path ? @$path : $path;
-        if (@paths) {
-            for ( @paths ) {
+        if ($path && @$path) {
+            for ( @$path ) {
+                #if (ref $_) {
+                #    my $foo = basename $$_;
+                #    my ($dir, $f) = (dirname($$_), basename($$_));
+                #    my $found = 0;
+                #    while (defined $f) {
+                #        ($dir, $f) = (dirname($dir), basename($dir));
+                #        #sleep 1;
+                #        my $fp = File::Spec->catfile( $dir, $filename );
+                #        #warn __PACKAGE__.':'.__LINE__.": fp=$fp\n";
+                #        if (-f $fp) {
+                #            local $" = "\\";
+                #            $PATHS{"@$path"}->{$filename} = $fp;
+                #            return $fp;
+                #        }
+                #        last if $dir eq '.';
+                #    }
+                #}
                 my $fp = File::Spec->catfile( $_, $filename );
-                return $fp if -f $fp;
+                if (-f $fp) {
+                    local $" = "\\";
+                    $PATHS{"@$path"}->{$filename} = $fp;
+                    return $fp;
+                }
             }
         }
-        else {
-            return $filename if -f $filename;
+        elsif (-f $filename) {
+            $PATHS{''}->{$filename} = $filename;
+            return $filename;
         }
 
         # TODO - bug with scalarref
@@ -689,12 +795,25 @@ sub _check_deprecated_args {
     }
 }
 
-sub init {
-    my ( $self, %args ) = @_;
+sub init_cache {
+    my ($self, $args) = @_;
+    if (exists $args->{cache_dir}) {
+        # will soon be deprecated
+        $args->{file_cache_dir} = $args->{cache_dir};
+        unless (exists $args->{file_cache}) {
+            # warn in future versions
+            $args->{file_cache} = 1;
+        }
+    }
+    $self->set_cache_dir($args->{file_cache_dir}) if $args->{file_cache};
     my $cachedir = $self->get_cache_dir;
-    if (defined $cachedir and not -d $cachedir) {
+    if ($args->{file_cache} and defined $cachedir and not -d $cachedir) {
         croak "Cachedir '$cachedir' does not exist";
     }
+}
+
+sub init {
+    my ( $self, %args ) = @_;
     my %defaults = (
 
         # defaults
@@ -702,12 +821,15 @@ sub init {
         loop_context_vars      => 0,
         case_sensitive         => $CASE_SENSITIVE_DEFAULT,
         debug                  => $DEBUG_DEFAULT,
+        debug_file             => 0,
+        objects                => 'strict',
         out_fh                 => 0,
         global_vars            => 0,
         default_escape         => $DEFAULT_ESCAPE,
         default_path           => PATH_DEREF,
         use_query              => $DEFAULT_QUERY,
         use_perl               => 0,
+        open_mode              => '',
         %args,
     );
     $self->set_loop_context(1) if $args{loop_context_vars};
@@ -715,6 +837,10 @@ sub init {
     $self->set_default_escape( $defaults{default_escape} );
     $self->set_default_path( $defaults{default_path} );
     $self->set_use_query( $defaults{use_query} );
+    if ($defaults{open_mode}) {
+        $defaults{open_mode} =~ s/^[<>]//; # <:utf8
+    }
+    $self->set_open_mode( $defaults{open_mode} );
     $self->set_search_path( $defaults{search_path_on_include} );
     $self->set_includes({});
     if ( $args{filter} ) {
@@ -723,6 +849,11 @@ sub init {
             HTML::Template::Compiled::Filter->new( $args{filter} ) );
     }
     $self->set_debug( $defaults{debug} );
+    $self->set_debug_file( $defaults{debug_file} );
+    $self->set_objects( $defaults{objects} );
+    if ($defaults{objects} and $defaults{objects} eq 'nostrict') {
+        require Scalar::Util;
+    }
     $self->set_out_fh( $defaults{out_fh} );
     $self->set_global_vars( $defaults{global_vars} );
     my $tagstyle = $args{tagstyle};
@@ -753,27 +884,37 @@ sub init {
     my $compiler = $self->compiler_class->new;
     $self->set_compiler($compiler);
     if ($defaults{plugin}) {
-        for my $plug (ref $defaults{plugin} eq 'ARRAY'
-            ? @{ $defaults{plugin} }
-            : $defaults{plugin}
-        ) {
-            if ($plug =~ m/^::/) {
-                $plug = "HTML::Template::Compiled::Plugin$plug";
-            }
+        my @plugins = ref $defaults{plugin} eq 'ARRAY' ? @{ $defaults{plugin} } : $defaults{plugin};;
+        $self->init_plugins(@plugins);
+        $self->set_plugins(\@plugins);
+    }
+}
+
+sub init_plugins {
+    my ($self, @plugins) = @_;
+    my $parser = $self->get_parser;
+    my $compiler = $self->get_compiler;
+    for my $plug (@plugins) {
+        if (ref $plug) {
+        }
+        elsif ($plug =~ m/^::/) {
+            $plug = "HTML::Template::Compiled::Plugin$plug";
+        }
+        unless ($plug->can('register')) {
             eval "require $plug";
             if ($@) {
                 carp "Could not load plugin $plug\n";
             }
-            my $actions = $self->get_plugin_actions($plug);
-            if (my $tagnames = $actions->{tagnames}) {
-                $parser->add_tagnames($tagnames);
-            }
-            if (my $escape = $actions->{escape}) {
-                $compiler->add_escapes($escape);
-            }
-            if (my $tags = $actions->{compile}) {
-                $compiler->set_tags($tags);
-            }
+        }
+        my $actions = $self->get_plugin_actions($plug);
+        if (my $tagnames = $actions->{tagnames}) {
+            $parser->add_tagnames($tagnames);
+        }
+        if (my $escape = $actions->{escape}) {
+            $compiler->add_escapes($escape);
+        }
+        if (my $tags = $actions->{compile}) {
+            $compiler->add_tags($tags);
         }
     }
 }
@@ -786,20 +927,28 @@ sub init {
         $plugins = [$plugins] unless ref $plugins eq 'ARRAY';
         for my $plug (@$plugins) {
             my $actions = $plug->register;
-            $classes->{$plug} = $actions;
+            $classes->{ref $plug || $plug} = $actions;
         }
     }
 
     sub get_plugin_actions {
         my ($self, $pclass) = @_;
-        return $classes->{$pclass};
+        return $classes->{ref $pclass || $pclass};
     }
 }
     
 
 sub _readfile {
     my ( $self, $file ) = @_;
-    open my $fh, $file or die "Cannot open '$file': $!";
+    my $open_mode = $self->get_open_mode;
+    my $fh;
+    if ($] < 5.007001) {
+        open $fh, $file or die "Cannot open '$file': $!";
+    }
+    else {
+        $open_mode = '' unless length $open_mode;
+        open $fh, "<$open_mode", $file or die "Cannot open '$file': $!";
+    }
     local $/;
     my $text = <$fh>;
     return $text;
@@ -902,8 +1051,11 @@ sub try_global {
 # end ugly code, phooey
 
 # returns if the var is valid
+# fix 2007-07-23: HTML::Template allows every character
+# although the documentation says it doesn't.
 sub validate_var {
-    return $_[1] !~ tr#a-zA-Z0-9._[]/-##c;
+    return 1;
+    #return $_[1] !~ tr{a-zA-Z0-9._[]/#-}{}c;
 }
 
 sub escape_filename {
@@ -971,10 +1123,15 @@ sub new_from_object {
     return $new;
 }
 
+sub prepare_for_cache {
+    my ($self) = @_;
+    $self->clear_params;
+}
+
 sub preload {
     my ( $class, $dir ) = @_;
     opendir my $dh, $dir or die "Could not open '$dir': $!";
-    my @files = grep { m/\.pl$/ } readdir $dh;
+    my @files = grep { m/\.pl|\.storable$/ } readdir $dh;
     closedir $dh;
     my $loaded = 0;
     for my $file (@files) {
@@ -1061,7 +1218,7 @@ sub query {
     # use query-like behaviour.
     return unless defined wantarray();
     #print STDERR "query(@_)\n";
-    my $info = $self->get_use_query
+    my $info = $self->get_parse_tree
         or do {
             $self->_error_no_query();
             return;
@@ -1076,7 +1233,7 @@ sub query {
     $tags = [$tags] unless ref $tags eq 'ARRAY';
     my $includes = $self->get_includes;
     my %include_info = map {
-        $includes->{$_}->[1] => $includes->{$_}->[2]->get_use_query;
+        $includes->{$_}->[1] => $includes->{$_}->[2]->get_parse_tree;
     } keys %{ $includes };
     for my $tag (@$tags) {
         my $value;
@@ -1246,6 +1403,29 @@ sub popGlobalstack {
     }
 }
 
+
+{
+    my $loaded = 0;
+    my $error = 0;
+    sub require_storable {
+        return 1 if $loaded;
+        return 0 if $error;
+        eval {
+            require Storable;
+        };
+        if ($@) {
+            $error = 1;
+            return 0;
+        }
+        eval "use B::Deparse 0.61";
+        if ($@) {
+            $error = 1;
+            return 0;
+        }
+        return 1;
+    }
+}
+
 my $version_pod = <<'=cut';
 =pod
 
@@ -1255,14 +1435,15 @@ HTML::Template::Compiled - Template System Compiles HTML::Template files to Perl
 
 =head1 VERSION
 
-$VERSION = "0.83_01"
+$VERSION = "0.91_001"
 
 =cut
 
 sub __test_version {
     my $v = __PACKAGE__->VERSION;
-    return 1 if $version_pod =~ m/VERSION.*\Q$v/;
-    return;
+    my ($v_test) = $version_pod =~ m/VERSION\s*=\s*"(.+)"/m;
+    no warnings;
+    return $v == $v_test ? 1 : 0;
 }
 
 1;
@@ -1297,6 +1478,9 @@ __END__
 
 For a quick reference, see L<HTML::Template::Compiled::Reference>.
 
+As the basic features work like in L<HTML::Template>, please get familiar
+with this documentation before.
+
 HTML::Template::Compiled (HTC) does not implement all features of
 L<HTML::Template>, and
 it has got some additional features which are explained below:
@@ -1308,20 +1492,21 @@ for what you need to know if you want (almost) the same behaviour). Internally
 it works different, because it turns the template into perl code,
 and once that is done, generating the output is much faster than with
 HTML::Template (3-7 times at the moment, depending on the options you use (see
-L<"Benchmarks"> for some examples), when both are run with loop_context_vars 0.
+L<"BENCHMARKS"> for some examples), when both are run with loop_context_vars 0.
 It also can generate perl files so that
 the next time the template is loaded it doesn't have to be parsed again. The best
 performance gain is probably reached in applications running under mod_perl, for example.
 
-If you don't use memory caching (e.g. CGI environment), HTC will be even
-slower than H::T.
-If you don't use caching at all (e.g. CGI environment without file caching),
-HTC will be much slower than H::T and TT.
+If you don't use any caching HTC will be very slow, slower than TT. Also
+with file caching but without memory caching it's the slowest templating
+module I know. With memory caching, though, it is one of the fastest,
+even faster sometimes (depending on options and template size) than
+C modules.
 
 You might want to use L<HTML::Template::Compiled::Lazy> for CGI environments
 as it doesn't parse the template before calling output. But note that HTC::Lazy
-is still in development; there might be bugs with certain combinations
-of includes and file caching.
+isn't much tested, and I don't use it myself, so there's a lack of experience.
+If you use it and have problems, please report.
 
 HTC will use a lot of memory because it keeps all template objects in memory.
 If you are on mod_perl, and have a lot of templates, you should preload them at server
@@ -1381,7 +1566,8 @@ use option case_sensitive => 0 to use this feature (slow down)
 
 =item C<query>
 
-Has a bug (doesn't return parameters in included files)
+Has a bug (doesn't return parameters in included files of included files).
+I'm working on that.
 
 =back
 
@@ -1395,6 +1581,10 @@ What can HTC do for you additionally to HTML::Template?
 
 No need to have cascading "if-else-if-else"s
 
+=item TMPL_EACH
+
+Iterate over a hash.
+
 =item TMPL_WITH
 
 see L<"TMPL_WITH">
@@ -1411,6 +1601,23 @@ see L<"TMPL_COMMENT">, L<"TMPL_NOPARSE">, L<"TMPL_VERBATIM">
 
 Additional loop variable (C<__counter__ -1>)
 
+=item C<__break__>
+
+Additional loop variable (see L<"TMPL_LOOP">)
+
+=item C<__filename__>, C<__filenameshort__> (since 0.91_001)
+
+Insert the template filename for debugging:
+
+    <%= __filename__ %>
+    <%= __filenameshort__ %>
+
+will turn out as:
+    templates/path/file.html
+    path/file.html
+
+See also option debug_file in L<"OPTIONS"> for adding the filename globally.
+
 =item TMPL_SWITCH, TMPL_CASE
 
 see L<"TMPL_SWITCH">
@@ -1419,6 +1626,28 @@ see L<"TMPL_SWITCH">
 
 Include perl code in your template. See L<"TMPL_PERL">
 
+=item Chomp
+
+Experimental feature, added in version 0.89. By using special tags the
+newlines before and/or after the tags will be deleted.
+I'm not sure about the syntax, so this might change. I'd be very glad about
+comments. Example:
+
+    my $htc = HTML::Template::Compiled->new(
+        ...
+        tagstyle => [qw/+asp_chomp/], # classic_chomp tt_chomp comment_chomp
+    );
+    template:
+    <+-%= foo %>
+    bar
+    <-+%= baz %>
+    is the same as
+    <%= foo %>bar<%= baz %>
+
+So C<+-> means, leave the newline in front alone, but chomp after it,
+C<-+> is the opposite, C<--> chomps both and C<++> is just a no-op and
+behaves like a normal tag.
+
 =item Generating perl code
 
 See L<"IMPLEMENTATION">
@@ -1534,6 +1763,16 @@ default is now 0, like in HTML::Template. Set it to 1 by
 default is 0 (off). Set it via
     HTML::Template::Compiled->UseQuery(1);
 
+=item open_mode
+
+If you want to have your templates read in utf-8, use
+
+    open_mode => ':utf8',
+
+as an option.
+
+In the previous version, it was '<:utf8'. This is deprecated.
+
 =back
 
 To be compatible in all of the above options all use:
@@ -1548,6 +1787,10 @@ which is the default but depending on user wishes that might change.
 
 =head3 Different behaviour from HTML::Template
 
+=over 4
+
+=item Arrayrefs
+
 At the moment this snippet
 
   <tmpl_if arrayref>true<tmpl_else>false</tmpl_if arrayref>
@@ -1561,12 +1804,33 @@ array is true if it has content, in HTC it's true if it (the reference) is
 defined. I'll try to find a way to change that behaviour, though that might
 be for the cost of speed.
 
+As of L<HTML::Template::Compiled> 0.85 you can use this syntax:
+
+    <tmpl_if arrayref# >true<tmpl_else>false</tmpl_if >
+
 In L<HTML::Template::Compiled::Classic> 0.04 it works as in HTML::Template.
 
+=item Searching the path
+
+In HTML::Template, if you have a file a/b/c/d/template.html and in
+that template you do an include of include.html, and include.html
+is in /a/b/include.html, HTML::Template will find it. As this
+wasn't so clear to me when reading the docs, I implemented
+this differently. You'd either have to include ../../include.html,
+or you should set search_path_on_include to 1 and include a/b/include.html.
+
+If you really need this feature, write me. I'm still thinking of how
+I would implement this, and I don't like it much, because it
+seems to me like a global_vars for filenames, and I don't like
+global_vars =)
+
+=back
+
 =head2 ESCAPING
 
 Like in HTML::Template, you have C<ESCAPE=HTML>, C<ESCAPE=URL> and C<ESCAPE_JS>.
-(C<ESCAPE=1> won't follow!  It's old and ugly...)
+C<ESCAPE=HTML> will only escape '"&<>. If you want to escape more, use
+C<ESCAPE=HTML_ALL>.
 Additionally you have C<ESCAPE=DUMP>, which by default will generate a Data::Dumper output.
 
 You can also chain different escapings, like C<ESCAPE=DUMP|HTML>.
@@ -1818,6 +2082,34 @@ Also you can give the current item an alias. See L<"ALIAS">. I also would like
 to add a loop_context variable C<__current__>, if that makes sense.
 Seems more readable to non perlers than C<_>.
 
+The LOOP tag allows you to define a JOIN attribute:
+
+ <tmpl_loop favourite_colors join=", ">
+  <tmpl_var _ >
+ </tmpl_loop>
+
+This will output something like C<blue, pink, yellow>.
+This is easier than doing:
+
+ <tmpl_loop favourite_colors>
+ <tmpl_unless __first__>, </tmpl_unless>
+  <tmpl_var _ >
+ </tmpl_loop>
+
+The C<LOOP>, C<WHILE> and C<EACH> tags allow you to define a BREAK attribute:
+
+ <tmpl_loop bingo break="3"> <tmpl_var _ ><if __break__>\n</if></tmpl_loop>
+
+    $htc->param(bingo => [qw(X 0 _ _ X 0 _ _ X)]);
+
+outputs
+
+    X 0 _
+    _ X 0
+    _ _ X
+
+So specifying BREAK=3 sets __break__ to 1 every 3rd loop iteration.
+
 =head2 TMPL_WHILE
 
 Useful for iterating, for example over database resultsets.
@@ -1921,10 +2213,20 @@ Path to template files
 Search the list of paths specified with C<path> when including a template.
 Default is 0
 
-=item cache_dir
+=item file_cache
+
+Set to 1 if you want to use file caching and specify the path
+with file_cache_dir.
+
+=item file_cache_dir
 
 Path to caching directory (you have to create it before)
 
+=item cache_dir
+
+Replaced by file_cache_dir like in L<HTML::Template>. Will be deprecated
+in future versions.
+
 =item cache
 
 Is 1 by default. If set to 0, no memory cacheing is done. Only recommendable if
@@ -1992,6 +2294,36 @@ browse through the stack.
 Now everything will be escaped for HTML unless you explicitly specify C<ESCAPE=0> (no escaping)
 or C<ESCAPE=URL>.
 
+=item debug_file (fixed) (since 0.91_001)
+
+Additionally to the context_vars __filename__ and __filenameshort__ you
+can enable filename debugging globally.
+
+If the option is set to 'start', at the start of every template will be added:
+    <!-- start templates/path/filename.html -->
+
+If set to 'end', at the end will be added:
+    <!-- end templates/path/filename.html -->
+
+If set to 'start,end', both coments will be added.
+
+If set to 'start,short', 'end,short' or 'start,end,short' the path
+to the templates will be stripped:
+    <!-- start path/filename.html -->
+    <!-- end path/filename.html -->
+
+=item objects (fixed) (since 0.91_001)
+
+if set to true, you can use method calls like
+    <%= object.method %>
+
+Default is 'strict' (true).
+If set to 'strict', the method will be called if we have an object, otherwise
+it's treated as a hash lookup. If the method doesn't exist, it dies.
+If set to 'nostrict', the method will be called only if the object 'can' do the
+method, otherwise it will return undef (this will need Scalar::Util).
+If set to 0, no method calls are allowed.
+
 =item deref (fixed)
 
 Deprecated. Please inherit and overwrite method 'deref'. See L<"INHERITANCE">
@@ -2258,6 +2590,14 @@ cache directory. See L<"PRECOMPILE">.
 
 Empty all parameters.
 
+=item get_plugin
+
+    my $plugin = $htc->get_plugin('Name::of::plugin');
+
+Returns the plugin object of that classname. If the plugin is only a string
+(the classname itself), it returns this string, so this method is only
+useful for plugin objects.
+
 =back
 
 =head1 EXPORT
@@ -2437,8 +2777,8 @@ Bjoern Kriews for original idea and contributions
 
 Special Thanks to Sascha Kiefer - he finds all the bugs!
 
-Ronnie Neumann, Martin Fabiani, Kai Sengpiel, Jan Willamowius, Justin Day
-for ideas, beta-testing and patches
+Ronnie Neumann, Martin Fabiani, Kai Sengpiel, Jan Willamowius, Justin Day,
+Steffen Winkler for ideas, beta-testing and patches
 
 perlmonks.org and perl-community.de for everyday learning
 
